 /**************************************************************************
 *   Copyright (C) 2005 by Vlad Svetlichny                                 *
 *   vladsv@inbox.ru                                                       *
 *   Distributed under GPL                                                 *
 ***************************************************************************/
#include "bookmarksb.h"
#include "addbookmarkdlg.h"
#include "tab.h"
#include <qlayout.h>
#include <qtabwidget.h>
#include <qmessagebox.h>
#include "kniga.h"
#include "settings.h"

BookmarkSB::BookmarkSB(QWidget *parent, Kniga *kkniga)
 : QWidget(parent)
{
    fkniga=kkniga;
    
    icoAdd = QPixmap::fromMimeSource("add.png");
    icoDelete = QPixmap::fromMimeSource("remove.png");
    icoReset = QPixmap::fromMimeSource("reset.png");
    icoRename = QPixmap::fromMimeSource("rename.png");
    
    lv=new QListBox(this);
    btnAdd=new QPushButton(this);
    btnDelete=new QPushButton(this);
    btnReset=new QPushButton(this);
    btnRename=new QPushButton(this);
    
    btnAdd->setPixmap(icoAdd);
    btnDelete->setPixmap(icoDelete);
    btnReset->setPixmap(icoReset);
    btnRename->setPixmap(icoRename);
    
    QVBoxLayout *vbox=new QVBoxLayout(this,0,5);
    vbox->addWidget(lv,50);
    QHBoxLayout *hbox=new QHBoxLayout(vbox,5);
    hbox->addWidget(btnAdd);
    hbox->addWidget(btnDelete);
    hbox->addWidget(btnReset);
    hbox->addWidget(btnRename);
    
    connect(btnAdd,SIGNAL(clicked()),SLOT(add()));
    connect(btnDelete,SIGNAL(clicked()),SLOT(remove()));
    connect(btnReset,SIGNAL(clicked()),SLOT(reset()));
    connect(btnRename,SIGNAL(clicked()),SLOT(rename()));
    connect(lv,SIGNAL(doubleClicked(QListBoxItem *)),SLOT(open(QListBoxItem *)));
    
    listReload();
}

void BookmarkSB::add()
{
    QTabWidget *tw=fkniga->tw;
    if (tw->count())
    {
    QString name, file=((tab*)(tw->currentPage()))->fileName,
    enc=((tab*)(tw->currentPage()))->charset;
    int position;
    tab *ttab=(tab*)(tw->currentPage());
    AddBookmarkDlg *dlg=new AddBookmarkDlg(this,ttab,&name,&position,FALSE);
    int r=dlg->exec();
    if (r) return;
    delete dlg;
    if (name.isEmpty()) return;
    (*fkniga->bm)[name]=BookmarkEntry(file,position,enc);
    fkniga->changeBookmarksMenu();
    listReload();
    }
}

void BookmarkSB::remove()
{
    int j;
    if (fkniga->settings->confirmOnCloseAll){
       j=QMessageBox::question(this,tr("Delete this bookmark"),tr("Are you sure?"),
               tr("Delete"),tr("Cancel"));
       if (j==1) return;
   }
    if (lv->count())
    {
        QString s=lv->currentText();
        (*fkniga->bm).erase(s);
        fkniga->changeBookmarksMenu();
        listReload();
    }
}

void BookmarkSB::reset()
{
    QTabWidget *tw=fkniga->tw;
    if (tw->count())
    {
        tab *t=(tab*)(tw->currentPage());
        QString s=lv->currentText(),
        file=t->fileName,
        enc=t->charset;
        int position=t->paragraphAt(t->XY);
        (*fkniga->bm).replace(s,BookmarkEntry(file,position,enc));
        fkniga->changeBookmarksMenu();
    }
}

void BookmarkSB::rename()
{
    if (lv->count())
    {
        QString name=lv->currentText();
        QString file=(*fkniga->bm)[name].file,
        enc=(*fkniga->bm)[name].enc;
        int position=(*fkniga->bm)[name].position;
        QString oldname=name;
        AddBookmarkDlg *dlg=new AddBookmarkDlg(this,0,&name,&position,TRUE);
        if (dlg->exec()) 
        {
            delete dlg;
            return;
        }    
        delete dlg;
        if (name.isEmpty()) return;
        (*fkniga->bm).erase(oldname);
        (*fkniga->bm)[name]=BookmarkEntry(file,position,enc);
        fkniga->changeBookmarksMenu();
        listReload();
    }
}

void BookmarkSB::listReload()
{
    lv->clear();
    for ( QMap<QString, BookmarkEntry>::Iterator it = (fkniga->bm)->begin();
    it != (fkniga->bm)->end(); ++it ) lv->insertItem(it.key());
}

void BookmarkSB::open(QListBoxItem *lbi)
{
    QString s=lbi->text();
    fkniga->load((*fkniga->bm)[s].file,(*fkniga->bm)[s].position,
    (*fkniga->bm)[s].enc);
}
