 /***************************************************************************
 *   Copyright (C) 2004 by Vlad Svetlichny                                 *
 *   vladsv@inbox.ru                                                       *
 *   Distributed under GPL                                                 *
 ***************************************************************************/
#include "find.h"
#include "tab.h"
#include <qcheckbox.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qmessagebox.h>

Find::Find(QWidget *parent, tab *ttab, const char *name)
 : QDialog(parent, name)
{
    setCaption(tr("Find"));
    
    QLabel *label = new QLabel(tr("Find &what:"), this); 
    lineEdit = new QLineEdit(this); 
    label->setBuddy(lineEdit); 
    wholeWordBox = new QCheckBox(tr("&Whole words only"), this);
    caseCheckBox = new QCheckBox(tr("Match &case"), this); 
    backwardCheckBox = new QCheckBox(tr("Search &backward"), this); 
    findButton = new QPushButton(tr("&Find"), this); 
    findButton->setDefault(true); 
    findNextButton=new QPushButton(tr("Find &next"), this);
    QPushButton *closeButton = new QPushButton(tr("Close"), this);      
    
    QHBoxLayout *topLeftLayout = new QHBoxLayout; 
    topLeftLayout->addWidget(label); 
    topLeftLayout->addWidget(lineEdit); 
    QVBoxLayout *leftLayout = new QVBoxLayout; 
    leftLayout->addLayout(topLeftLayout);
    leftLayout->addWidget(wholeWordBox);  
    leftLayout->addWidget(caseCheckBox); 
    leftLayout->addWidget(backwardCheckBox); 
    QVBoxLayout *rightLayout = new QVBoxLayout; 
    rightLayout->addWidget(findButton); 
    rightLayout->addWidget(findNextButton);
    rightLayout->addWidget(closeButton); 
    rightLayout->addStretch(1); 
    QHBoxLayout *mainLayout = new QHBoxLayout(this); 
    mainLayout->setMargin(11);
    mainLayout->setSpacing(6); 
    mainLayout->addLayout(leftLayout); 
    mainLayout->addLayout(rightLayout);
    
    connect(findButton, SIGNAL(clicked()), 
       this, SLOT(findAgain()));
    connect(findNextButton, SIGNAL(clicked()), 
       this, SLOT(findNext())); 
    connect(closeButton, SIGNAL(clicked()), 
       this, SLOT(close()));
    ttext=ttab;
    
    para=index=0;   
}


Find::~Find()
{   
}

void Find::findSome()
{
    QString allText=ttext->text(),
            findText=lineEdit->text();
    bool findBack=backwardCheckBox->isChecked(),
         caseSens=caseCheckBox->isChecked(),
         wholeWord=wholeWordBox->isChecked();
         
    bool isFind=ttext->find(findText,caseSens,wholeWord,!findBack,&para,&index);
    
    if (!isFind) {
                     QMessageBox::information(this, tr("Find"), 
                     tr("Cannot find %1 in this text").arg(findText));
                     findButton->setDefault(true);
                     findNextButton->setDefault(false);
                 }     
}

void Find::findAgain()
{
  bool findBack=backwardCheckBox->isChecked();
  if (findBack) {
      para=ttext->paragraphs()-1;
      index=ttext->paragraphLength(para)-1;
  }
  else para=index=0;
  findSome();
  findButton->setDefault(false);
  findNextButton->setDefault(true);
}

void Find::findNext()
{
  int max=ttext->paragraphLength(para)-1;
  bool findBack=backwardCheckBox->isChecked();
  if (index<max)
  {
      if (findBack) {index--;}
      else index++;
  }
  else
  {
      if (findBack) {para--;}
      else para++;
  };
  findSome();
}
