 /**************************************************************************
 *   Copyright (C) 2005 by Vlad Svetlichny                                 *
 *   vladsv@inbox.ru                                                       *
 *   Distributed under GPL                                                 *
 ***************************************************************************/
#include "kniga.h"
#include "config.h"
#include "tab.h"
#include "prefs.h"
#include "encselect.h"
#include "find.h"
#include "settings.h"
#include "addbookmarkdlg.h"
#include "sidebar.h"
#include "bookmarksb.h"

#include <qaction.h>
#include <qtimer.h>
#include <qimage.h>
#include <qpixmap.h>
#include <qtoolbar.h>
#include <qmenubar.h>
#include <qtextedit.h>
#include <qfiledialog.h>
#include <qstatusbar.h>
#include <qprinter.h>
#include <qapplication.h>
#include <qaccel.h>
#include <qpainter.h>
#include <qpaintdevicemetrics.h>
#include <qwhatsthis.h>
#include <qtabwidget.h>
#include <qdragobject.h>
#include <qmessagebox.h>
#include <qmap.h>
#include <qhbox.h>
#include <qsplashscreen.h>
#include <qprocess.h>
#include <qclipboard.h>

Kniga::Kniga()
    : QMainWindow( 0, "Kniga", WDestructiveClose )
{
    settings=new Settings();
    bm=&(settings->bm);
    settings->loadSettings();
    sbVisible=settings->sbVisible;
    
    scrollTimer=new QTimer(this);
    scrollTimer->start(settings->scroll);
    
    setCaption(tr("Kniga"));
    setIcon(QPixmap::fromMimeSource("kniga_icon.png"));
    printer = new QPrinter;

    QPixmap openIcon, saveIcon, printIcon, closeIcon, prevIcon, nextIcon,
    prefIcon, zoomInIcon, zoomOutIcon, findIcon, autoscrollIcon, fullIcon;
    openIcon = QPixmap::fromMimeSource("fileopen.png");
    closeIcon = QPixmap::fromMimeSource("fileclose.png");
    printIcon = QPixmap::fromMimeSource("fileprint.png");
    prevIcon = QPixmap::fromMimeSource("prevpage.png");    
    nextIcon = QPixmap::fromMimeSource("nextpage.png");
    prefIcon = QPixmap::fromMimeSource("configure.png");
    zoomInIcon = QPixmap::fromMimeSource("zoomin.png");
    zoomOutIcon = QPixmap::fromMimeSource("zoomout.png");
    findIcon = QPixmap::fromMimeSource("find.png");
    fullIcon = QPixmap::fromMimeSource("fullscreen.png");
    autoscrollIcon = QPixmap::fromMimeSource("autoscroll.png");

    QString fileOpenText = tr("<p><img source=\"fileopen.png\"> "
	         "Click this button to open a file in a <em>new tab</em>. <br>"
                 "You can also select the <b>Open</b> command "
                 "from the <b>File</b> menu.</p>"),
    
    filePrintText = tr("<p><img source=\"fileprint.png\">"
                 "Click this button to print the file you "
                 "are reading.\n You can also select the <b>Print</b> "
                 "command from the <b>File</b> menu.</p>"),

    prevTabText = tr("<p><img source=\"prevpage.png\">"
                 "Click this button to select the previous page.\n "
                 "You can also select the <b>Previous page</b> "
                 "command from the <b>Tabs</b> menu.</p>"),
		 
    nextTabText = tr("<p><img source=\"nextpage.png\">"
                 "Click this button to select the next page.\n "
                 "You can also select the <b>Next page</b> "
                 "command from the <b>Tabs</b> menu.</p>"),
    
    closeText = tr("<p><img source=\"fileclose.png\">"
                 "Click this button to close the current page.\n "
                 "You can also select the <b>Close current page</b>"
                 "command from the <b>Tabs</b> menu.</p>"),
    prefText = tr("<p><img source=\"configure.png\">"
                 "Click this button to configure the program.\n "
                 "You can also select the <b>Preferences</b> "
                 "command from the <b>File</b> menu.</p>");

    fileOpenAct=new QAction(openIcon,tr("&Open..."),CTRL+Key_O,this),
    filePrintAct=new QAction(printIcon,tr("&Print..."),CTRL+Key_P,this),
    filePrefAct=new QAction(prefIcon,tr("&Preferences"),0,this),
    fileQuitAct=new QAction(tr("&Quit"),CTRL+Key_Q,this),
    viewFullAct=new QAction(fullIcon,tr("&Fullscreen mode"),Key_F11,this),
    viewZoomInAct=new QAction(zoomInIcon,tr("Zoom &in"),CTRL+Key_Plus,this),
    viewZoomOutAct=new QAction(zoomOutIcon,tr("Zoom &out"),CTRL+Key_Minus,this),
    viewSetEncAct=new QAction(tr("&Set encoding..."),0,this),
    viewAutoScrollAct=new QAction(autoscrollIcon,tr("&Autoscrolling"),Key_F2,this);
    searchFindAct=new QAction(findIcon,tr("&Find..."),CTRL+Key_F,this),
    tabNextAct=new QAction(nextIcon,tr("&Next page"),ALT+Key_Right,this),
    tabPrevAct=new QAction(prevIcon,tr("&Previous page"),ALT+Key_Left,this),
    tabCloseCurrentAct=new QAction(closeIcon,
            tr("&Close current page"),CTRL+Key_W,this),
    tabCloseAllAct=new QAction(tr("Close &all"),Key_Escape,this),
    sbShowAct=new QAction(tr("&Show sidebar"),Key_F9,this),
    sbBookmarksAct=new QAction(tr("&Bookmarks"),0,this),
    sbStatsAct=new QAction(tr("&Statistics"),0,this),
    bmAddAct=new QAction(tr("Add a &bookmark..."),CTRL+Key_B,this),
    tulzSpeakAct=new QAction(tr("Speak selected text"),0,this),
    helpAboutAct=new QAction(tr("&About"),Key_F1,this),
    helpAboutQtAct=new QAction(tr("About &Qt"),0,this),
    helpWhatsThisAct=new QAction(tr("What's &This"),SHIFT+Key_F1,this);
    helpReadmeAct=new QAction(QString("README"),0,this);
    helpAuthorsAct=new QAction(QString("Authors"),0,this);
    helpChangeLogAct=new QAction(QString("ChangeLog"),0,this);
    helpCopyingAct=new QAction(QString("Copying"),0,this);
    helpToDoAct=new QAction(QString("TODO"),0,this);
    
            
    viewAutoScrollAct->setToggleAction(TRUE);
    viewAutoScrollAct->setOn(settings->autoscroll);
    sbShowAct->setToggleAction(TRUE);
    sbShowAct->setOn(sbVisible);
    viewFullAct->setToggleAction(TRUE);
    viewFullAct->setOn(settings->fullscreen);
    
    connect(fileOpenAct,SIGNAL(activated()),SLOT(choose()));
    connect(filePrintAct,SIGNAL(activated()),SLOT(print()));
    connect(filePrefAct,SIGNAL(activated()),SLOT(preferences()));
    connect(fileQuitAct,SIGNAL(activated()),qApp,SLOT(closeAllWindows()));
    connect(viewFullAct,SIGNAL(toggled(bool)),SLOT(fullScreen(bool)));
    connect(viewZoomInAct,SIGNAL(activated()),SLOT(zoomIn()));
    connect(viewZoomOutAct,SIGNAL(activated()),SLOT(zoomOut()));
    connect(viewAutoScrollAct,SIGNAL(toggled(bool)),SLOT(autoScroll(bool)));
    connect(viewSetEncAct,SIGNAL(activated()),SLOT(setEnc()));
    connect(searchFindAct,SIGNAL(activated()),SLOT(findClicked()));
    connect(tabNextAct,SIGNAL(activated()),SLOT(nextTab()));
    connect(tabPrevAct,SIGNAL(activated()),SLOT(prevTab()));
    connect(tabCloseCurrentAct,SIGNAL(activated()),SLOT(closeTab()));
    connect(tabCloseAllAct,SIGNAL(activated()),SLOT(closeAll()));
    connect(sbShowAct,SIGNAL(toggled(bool)),SLOT(sbShow(bool)));
    connect(sbBookmarksAct,SIGNAL(activated()),SLOT(sbBookmarks()));
    connect(sbStatsAct,SIGNAL(activated()),SLOT(sbStatistics()));
    connect(bmAddAct,SIGNAL(activated()),SLOT(addBookmark()));
    connect(tulzSpeakAct,SIGNAL(activated()),SLOT(sayText()));
    connect(helpAboutAct,SIGNAL(activated()),SLOT(about()));
    connect(helpAboutQtAct,SIGNAL(activated()),SLOT(aboutQt()));
    connect(helpWhatsThisAct,SIGNAL(activated()),SLOT(whatsThis()));    
    connect(helpReadmeAct,SIGNAL(activated()),SLOT(showReadme()));
    connect(helpAuthorsAct,SIGNAL(activated()),SLOT(showAuthors()));
    connect(helpToDoAct,SIGNAL(activated()),SLOT(showToDo()));
    connect(helpChangeLogAct,SIGNAL(activated()),SLOT(showChangeLog()));
    connect(helpCopyingAct,SIGNAL(activated()),SLOT(showCopying()));

    fileOpenAct->setWhatsThis(fileOpenText);
    filePrintAct->setWhatsThis(filePrintText );
    filePrefAct->setWhatsThis(prefText );
    tabPrevAct->setWhatsThis(prevTabText );
    tabNextAct->setWhatsThis(nextTabText );
    tabCloseCurrentAct->setWhatsThis(closeText );
    
    QToolBar *fileTools = new QToolBar(this);
    fileTools->setLabel(tr("File Operations"));
    fileOpenAct->addTo(fileTools);
    filePrintAct->addTo(fileTools);
    filePrefAct->addTo(fileTools);
    searchFindAct->addTo(fileTools);                       
    (void)QWhatsThis::whatsThisButton( fileTools );
    
    QToolBar *viewTools = new QToolBar(this);
    viewTools->setLabel(tr("View"));
    viewZoomInAct->addTo(viewTools);
    viewZoomOutAct->addTo(viewTools);
    viewFullAct->addTo(viewTools);
    viewAutoScrollAct->addTo(viewTools);
    
    QToolBar *tabTools = new QToolBar(this);
    tabTools->setLabel(tr("Pages"));
    tabCloseCurrentAct->addTo(tabTools);
    tabPrevAct->addTo(tabTools);
    tabNextAct->addTo(tabTools);

    QPopupMenu *file = new QPopupMenu(this);
    menuBar()->insertItem(tr("&File"),file);    
    file->insertTearOffHandle();
    fileOpenAct->addTo(file);
    filePrintAct->addTo(file);
    file->insertItem(tr("&Recent Files"),recent=createRecentMenu());
    file->insertSeparator();
    filePrefAct->addTo(file);
    file->insertSeparator();
    fileQuitAct->addTo(file);

    QPopupMenu *view=new QPopupMenu(this);
    menuBar()->insertItem(tr("&View"),view );
    view->insertTearOffHandle();
    viewFullAct->addTo(view);
    viewZoomInAct->addTo(view);
    viewZoomOutAct->addTo(view);
    viewAutoScrollAct->addTo(view);
    viewSetEncAct->addTo(view);

    QPopupMenu *search=new QPopupMenu(this);
    menuBar()->insertItem(tr("&Search"),search);
    search->insertTearOffHandle();
    searchFindAct->addTo(search);

    QPopupMenu *tabs = new QPopupMenu(this);
    menuBar()->insertItem(tr("&Tabs"),tabs);
    tabs->insertTearOffHandle();
    tabNextAct->addTo(tabs);
    tabPrevAct->addTo(tabs);
    tabCloseCurrentAct->addTo(tabs);
    tabCloseAllAct->addTo(tabs);
                           
    sbMenu= new QPopupMenu(this);
    menuBar()->insertItem( tr("&Sidebar"), sbMenu );    
    sbMenu->insertTearOffHandle();
    sbShowAct->addTo(sbMenu);
    sbMenu->insertSeparator();
    sbBookmarksAct->addTo(sbMenu);
    sbStatsAct->addTo(sbMenu);

    createBookmarksMenu();
    menuBar()->insertItem( tr("&Bookmarks"),bookmenu);
    menuBar()->insertSeparator();
    
    QPopupMenu *tulz=new QPopupMenu(this);
    menuBar()->insertItem(tr("&Tools"),tulz);
    tulz->insertTearOffHandle();
    tulzSpeakAct->addTo(tulz);
    
    QPopupMenu *help=new QPopupMenu(this);
    menuBar()->insertItem(tr("&Help"),help);
    help->insertTearOffHandle();
    helpAboutAct->addTo(help);
    helpAboutQtAct->addTo(help);
    help->insertSeparator();
    helpWhatsThisAct->addTo(help);
    help->insertSeparator();
    helpReadmeAct->addTo(help);
    helpAuthorsAct->addTo(help);
    helpChangeLogAct->addTo(help);
    helpCopyingAct->addTo(help);
    helpToDoAct->addTo(help);
    
    setAcceptDrops(true);

    QHBox *hbox=new QHBox(this);
    setCentralWidget(hbox);
    
    sb=new sidebar(hbox,this);
    tw = new QTabWidget(hbox);
    sb->setFixedWidth(settings->sbWidth);
    sb->setShown(sbVisible);
    
    sb->setCurrent(settings->sbCurrent);
    sb->changeBar(settings->sbCurrent);
    
    hbox->setMargin(5);
    hbox->setSpacing(5);
    statusBar()->message( tr("Welcome to Kniga!"));
    connect(tw, SIGNAL(currentChanged(QWidget*)),SLOT(changedTab()));
    resize( settings->winWidth, settings->winHeight );
    if (settings->fullscreen) showFullScreen();
    changedTab();
}


Kniga::~Kniga()
{
    delete printer;
    delete settings;
    delete scrollTimer;
}

void Kniga::choose()
{
    QString fn = QFileDialog::getOpenFileName( QString::null,
    tr("Text files (*.txt *.TXT);;"
    "HTML files (*.htm *.html *.HTM *.HTML);;"
    "MS Word files (*.doc *.DOC);;"
    "All files (*)"),
    this,
    tr("Open"),
    tr("Choose a file"));
    if ( !fn.isEmpty() )
	load( fn );
    else
	statusBar()->message( tr("Loading aborted"), 5000 );
}


void Kniga::load(QString FileName, int position, QString enc )
{
    tab *newtab= new tab(this, settings, FileName,position,enc);
    int c=tw->count();
    tw->insertTab(newtab,strippedName(FileName),c);
    tw->setCurrentPage(c);
    changedTab();
    statusBar()->message( tr("Loaded document %1").arg(FileName),5000);
    for (int i=4; i>0; i--) settings->recentFiles[i]=settings->recentFiles[i-1];
    settings->recentFiles[0]=FileName;
    changeRecentMenu();
}

void Kniga::print()
{
    if (!(tw->count())) return;
    tab* t=(tab*)(tw->currentPage());
// ###### Rewrite to use QSimpleRichText to print here as well
    const int Margin = 10;
    int pageNo = 1;

    if ( printer->setup(this) ) {		// printer dialog
	statusBar()->message( tr("Printing..."));
	QPainter p;
	if( !p.begin( printer ) )               // paint on printer
	    return;

	p.setFont(t->font() );
	int yPos	= 0;			// y-position for each line
	QFontMetrics fm = p.fontMetrics();
	QPaintDeviceMetrics metrics( printer ); // need width/height
						// of printer surface
	for( int i = 0 ; i < t->lines() ; i++ ) {
	    if ( Margin + yPos > metrics.height() - Margin ) {
	QString msg( "Printing (page " );
		msg += QString::number( ++pageNo );
		msg += ")...";
		statusBar()->message( msg );
		printer->newPage();		// no more room on this page
		yPos = 0;			// back to top of page
	    }
	    p.drawText( Margin, Margin + yPos,
			metrics.width(), fm.lineSpacing(),
			ExpandTabs | DontClip,
			t->text( i ) );
	    yPos = yPos + fm.lineSpacing();
	}
	p.end();				// send job to printer
	statusBar()->message( tr("Printing completed"), 5000 );
    } else {
	statusBar()->message( tr("Printing aborted"), 5000 );
    }
}

void Kniga::closeEvent( QCloseEvent* ce )
{
    if (!settings->confirmOnExit)
    {
        ce->accept();
        settings->winWidth=width();
        settings->winHeight=height();
        settings->fullscreen=isFullScreen();
        settings->sbCurrent=sb->currentIndex;
	settings->saveSettings();
        return;
    }
     
    switch( QMessageBox::information( this, tr("Close this window"),
				      tr("Are you sure?"),
				      tr("Quit"), tr("Cancel"), 0, 1 ) ) 
    {
    case 0:
	ce->accept();
        settings->winWidth=width();
        settings->winHeight=height();
        settings->fullscreen=isFullScreen();
	settings->saveSettings();
	break;
    case 1:
    default:
	ce->ignore();
	break;
    }
}


void Kniga::about()
{ 
    QSplashScreen *splash = new QSplashScreen(
        QPixmap::fromMimeSource("kniga.png"));
    splash->show();
}


void Kniga::aboutQt()
{
    QMessageBox::aboutQt( this, tr("Kniga"));
}

void Kniga::closeTab()
{
   tab *t=(tab *)(tw->currentPage());
   tw->removePage(t);
   delete t;
   changedTab();
}

void Kniga::changedTab()
{
   if (tw->count()) {
     setCaption(tr("Kniga - %1").arg(tw->tabLabel(tw->currentPage())));
     if (tw->currentPageIndex()==0) {
         tabPrevAct->setDisabled( TRUE );
     }
     else {
         tabPrevAct->setDisabled( FALSE ); 
     };
     if (tw->currentPageIndex()==(tw->count()-1)) {
         tabNextAct->setDisabled( TRUE );
     }
     else {
         tabNextAct->setDisabled( FALSE ); 
     };
     tabCloseCurrentAct->setDisabled( FALSE );
     searchFindAct->setDisabled( FALSE );
     filePrintAct->setDisabled( FALSE );
     viewZoomInAct->setDisabled( FALSE );
     viewZoomOutAct->setDisabled( FALSE );
     bmAddAct->setDisabled(FALSE);
   }
   else {
     setCaption(tr("Kniga"));
     tabCloseCurrentAct->setDisabled( TRUE );
     searchFindAct->setDisabled( TRUE );
     filePrintAct->setDisabled( TRUE );
     viewZoomInAct->setDisabled( TRUE );
     viewZoomOutAct->setDisabled( TRUE );
     tabNextAct->setDisabled(TRUE);
     tabPrevAct->setDisabled(TRUE);
     bmAddAct->setDisabled(TRUE);
   }
}

void Kniga::nextTab()
{
   int cnt=tw->count(), curr=tw->currentPageIndex();
   if (curr<cnt) {
   tw->setCurrentPage(curr+1);
   };
}

void Kniga::prevTab()
{
   int curr=tw->currentPageIndex();
   if (curr>0) {
   tw->setCurrentPage(curr-1);
   };  
}

void Kniga::preferences()
{
   prefs *pref=new prefs(this,settings);
   pref->exec();
   delete pref;
   for(int i=0; i<(tw->count()); i++)
   {
       ((tab*)(tw->page(i)))->recolor();
       ((tab*)(tw->page(i)))->refont();
   }
   scrollTimer->changeInterval(settings->scroll);
   sb->setFixedWidth(settings->sbWidth);
}

void Kniga::closeAll()
{
   int j;
   if (settings->confirmOnCloseAll){
       j=QMessageBox::question(this,tr("Close all pages"),tr("Are you sure?"),
               tr("Close"),tr("Cancel"));
       if (j==1) return;
   }

   for(int i=(tw->count())-1; i>=0; i--)
   {
       tab *t=(tab *)(tw->page(i));
       tw->removePage(t);
       delete t;
   };
   changedTab();
}

void Kniga::zoomIn()
{
   if (tw->count()) ((tab*)(tw->currentPage()))->zoomIn(5);
}

void Kniga::zoomOut()
{
   if (tw->count()) ((tab*)(tw->currentPage()))->zoomOut(5);
}

void Kniga::setEnc()
{
   if (tw->count())
   {
       EncSelect *enc=new EncSelect(this,(tab*)(tw->currentPage()),settings);
       enc->exec();
       delete enc;
   }
}

void Kniga::findClicked()
{  
   if (tw->count())
   {
      Find *dlg=new Find(this,(tab*)(tw->currentPage()));
      dlg->exec();
      delete dlg;
   }
}

QString Kniga::strippedName(QString fullFileName) 
{ 
   return QFileInfo(fullFileName).fileName();
}      

void Kniga::dragEnterEvent(QDragEnterEvent *event)
{
   event->accept(QUriDrag::canDecode(event));
} 
 
void Kniga::dropEvent(QDropEvent *event)
{
   QStringList fileNames; 
   if (QUriDrag::decodeLocalFiles(event, fileNames))
      load(fileNames[0]);
}

void Kniga::recfiles(int i)
{
    QString s=recent->text(i);
    int j=s.mid(1,1).toInt();
    load(settings->recentFiles[j-1]);
    
}

QPopupMenu* Kniga::createRecentMenu()
{
    QPopupMenu* recFiles=new QPopupMenu(this);
    
    for (int i=0; i<5; i++) {
     rec[i]=recFiles->insertItem(QString("&%1 ").arg(i+1)
     +strippedName(settings->recentFiles[i]), this, 
     SLOT(recfiles(int)));
    }

    return recFiles;

}

void Kniga::changeRecentMenu()
{
    for (int i=0; i<5; i++){
        recent->changeItem(rec[i],QString("&%1 ")
        .arg(i+1)+strippedName(settings->recentFiles[i]));
    }

}

void Kniga::bookmarksClick(int i)
{
    QString s=bookmenu->text(i);
    load((*bm)[s].file,(*bm)[s].position, (*bm)[s].enc);
    
}

void Kniga::bookmarksDelClick(int i)
{
    int j;   
    if (settings->confirmOnBmDel)
    {
       j=QMessageBox::question(this,tr("Delete this bookmark"),tr("Are you sure?"),
               tr("Delete"),tr("Cancel"));
       if (j==1) return;
    }
    QString s=bookdelmenu->text(i);
    (*bm).erase(s);
    changeBookmarksMenu();
    ((sidebar*)(sb))->bookmarksb->listReload();
}

void Kniga::bookmarksResetClick(int i)
{
    if (tw->count())
    {
        tab *t=(tab*)(tw->currentPage());
        QString s=bookresetmenu->text(i),
        file=t->fileName,
        enc=t->charset;
        int position=t->paragraphAt(t->XY);
        (*bm).replace(s,BookmarkEntry(file,position,enc));
        changeBookmarksMenu();
        ((sidebar*)(sb))->bookmarksb->listReload();
    }
}

void Kniga::bookmarksRenameClick(int i)
{
    QString name=bookrenamemenu->text(i);
    QString file=(*bm)[name].file,
    enc=(*bm)[name].enc;
    int position=(*bm)[name].position;
    QString oldname=name;
    AddBookmarkDlg *dlg=new AddBookmarkDlg(this,0,&name,&position,TRUE);
    int r=dlg->exec();
    if (r) return;
    delete dlg;
    if (name.isEmpty()) return;
    (*bm).erase(oldname);
    (*bm)[name]=BookmarkEntry(file,position,enc);
    changeBookmarksMenu();
    ((sidebar*)sb)->bookmarksb->listReload();
}

void Kniga::addBookmark()
{
    if (tw->count())
    {
    QString name, file=((tab*)(tw->currentPage()))->fileName,
    enc=((tab*)(tw->currentPage()))->charset;
    int position;
    tab *ttab=(tab*)(tw->currentPage());
    AddBookmarkDlg *dlg=new AddBookmarkDlg(this,ttab,&name,&position);
    int r=dlg->exec();
    if (r) return;
    delete dlg;
    if (name.isEmpty()) return;
    (*bm)[name]=BookmarkEntry(file,position,enc);
    changeBookmarksMenu();
    ((sidebar*)sb)->bookmarksb->listReload();
    }
}

void Kniga::createBookmarksMenu()
{
    bookmenu=new QPopupMenu(this);
    bookmenu->insertTearOffHandle();
    bmAddAct->addTo(bookmenu);
    
    bookdelmenu = new QPopupMenu(this);
    for ( QMap<QString, BookmarkEntry>::Iterator it = bm->begin();
    it != bm->end(); ++it ) {
      bookdelmenu->insertItem(it.key(),this,SLOT(bookmarksDelClick(int)));
    }
    
    bookmenu->insertItem(tr("&Delete bookmark"),bookdelmenu);
    
    bookresetmenu = new QPopupMenu(this);
    for ( QMap<QString, BookmarkEntry>::Iterator it = bm->begin();
    it != bm->end(); ++it ) {
      bookresetmenu->insertItem(it.key(),this,SLOT(bookmarksResetClick(int)));
    }
    
    bookmenu->insertItem(tr("&Set to current position"),bookresetmenu);
    
    bookrenamemenu = new QPopupMenu(this);
    for ( QMap<QString, BookmarkEntry>::Iterator it = bm->begin();
    it != bm->end(); ++it ) {
      bookrenamemenu->insertItem(it.key(),this,SLOT(bookmarksRenameClick(int)));
    }
    
    bookmenu->insertItem(tr("Re&name bookmark"),bookrenamemenu);
    bookmenu->insertSeparator();
    
    for ( QMap<QString, BookmarkEntry>::Iterator it = bm->begin();
    it != bm->end(); ++it ) {
      bookmenu->insertItem(it.key(),this,SLOT(bookmarksClick(int)));
    }
}

void Kniga::changeBookmarksMenu()
{
    bookmenu->clear();
    bookmenu->insertTearOffHandle();
    bmAddAct->addTo(bookmenu);
    
    bookdelmenu->clear();
    for ( QMap<QString, BookmarkEntry>::Iterator it = bm->begin();
    it != bm->end(); ++it ) {
      bookdelmenu->insertItem(it.key(),this,SLOT(bookmarksDelClick(int)));
    }
    bookmenu->insertItem(tr("&Delete bookmark"),bookdelmenu);
    
    bookresetmenu->clear();
    for ( QMap<QString, BookmarkEntry>::Iterator it = bm->begin();
    it != bm->end(); ++it ) {
      bookresetmenu->insertItem(it.key(),this,SLOT(bookmarksResetClick(int)));
    }
    bookmenu->insertItem(tr("&Set to current position"),bookresetmenu);
    
    bookrenamemenu->clear();
    for ( QMap<QString, BookmarkEntry>::Iterator it = bm->begin();
    it != bm->end(); ++it ) {
      bookrenamemenu->insertItem(it.key(),this,SLOT(bookmarksRenameClick(int)));
    }
    
    bookmenu->insertItem(tr("Re&name bookmark"),bookrenamemenu);
    bookmenu->insertSeparator();
    
    QMap<QString, BookmarkEntry>::Iterator it; 
    for ( it = bm->begin(); it != bm->end(); ++it ) {
      bookmenu->insertItem(it.key(),this,SLOT(bookmarksClick(int)));
    }
}

void Kniga::fullScreen(bool f)
{
   tab *t;
   for(int i=(tw->count())-1; i>=0; i--)
   {
       t=(tab *)(tw->page(i));
       t->savePosition();
   };
  if (!f)
  {
      showNormal();
      menuBar()->show();
//       fullBtn->setIconSet(fullIcon);
//       fullscreen=FALSE;
  }
  else
  {
      showFullScreen();
      menuBar()->hide();
//       fullBtn->setIconSet(noFullIcon);
//       fullscreen=TRUE;
  };
  for(int i=(tw->count())-1; i>=0; i--)
   {
       t=(tab *)(tw->page(i));
       t->loadPosition();
   };
}

void Kniga::sbShow(bool f)
{
    tab *t;
    for(int i=(tw->count())-1; i>=0; i--)
    {
       t=(tab *)(tw->page(i));
       t->savePosition();
    };
    sb->setShown(f);
    for(int i=(tw->count())-1; i>=0; i--)
    {
       t=(tab *)(tw->page(i));
       t->loadPosition();
    };
}

void Kniga::sbBookmarks()
{
    sb->setCurrent(0);
    sb->changeBar(0);
}

void Kniga::sbStatistics()
{
    sb->setCurrent(1);
    sb->changeBar(1);
}

void Kniga::scrolling()
{
//     QMessageBox::information(this,QString(""),QString(""));
    if (!tw->count()) return;
    ((tab*)(tw->currentPage()))->scrollBy(0,1);
}

void Kniga::autoScroll(bool f)
{
    if (f)
    {
        connect(scrollTimer,SIGNAL(timeout()),this,SLOT(scrolling()));
    }
    else
    {
        disconnect(scrollTimer,SIGNAL(timeout()),this,SLOT(scrolling()));
    }
}

void Kniga::showAuthors()
{
    load(QString(DOC_PATH)+QString("AUTHORS"));
}

void Kniga::showChangeLog()
{
    load(QString(DOC_PATH)+QString("ChangeLog"));
}

void Kniga::showCopying()
{
    load(QString(DOC_PATH)+QString("COPYING"));
}

void Kniga::showReadme()
{
    load(QString(DOC_PATH)+QString("README"));
}

void Kniga::showToDo()
{
    load(QString(DOC_PATH)+QString("TODO"));
}

void Kniga::sayText()
{
  if (tw->count())
  {
  (QApplication::clipboard())->setText(
      ((tab*)(tw->currentPage()))->selectedText());
  QProcess *p = new QProcess( QString("dcop"), this );
  p->addArgument(QString("kttsd"));
  p->addArgument(QString("kspeech"));
  p->addArgument(QString("speakClipboard"));
  if ( !p->start() ) {
    QMessageBox::critical( this,tr("Error"),tr("Can't connect to KTTSD!"));
    return;
  }
}
}
