 /**************************************************************************
 *   Copyright (C) 2005 by Vlad Svetlichny                                 *
 *   vladsv@inbox.ru                                                       *
 *   Distributed under GPL                                                 *
 ***************************************************************************/
#include "settings.h"
#include "config.h"
#include <qsettings.h>
#include <qfont.h>
#include <qcolor.h>

void Settings::loadSettings()
{
    QSettings settings;
    settings.setPath( "Vlad Svetlichny", "Kniga" );
    winWidth=settings.readNumEntry("/kniga/win_geometry/winWidth",730);
    sbWidth=settings.readNumEntry("/kniga/sidebar/width",200);
    winHeight=settings.readNumEntry("/kniga/win_geometry/winHeight",600);
    fullscreen=settings.readBoolEntry("/kniga/win_geometry/fullscreen",FALSE);
    sbVisible=settings.readBoolEntry("/kniga/sidebar/isVisible",TRUE);
    sbCurrent=settings.readNumEntry("/kniga/sidebar/currentWidget",0);
    font=QFont(
    settings.readEntry("/kniga/font/name","FreeMono"),
    settings.readNumEntry("/kniga/font/size",10),
    settings.readNumEntry("/kniga/font/weight",50),
    settings.readBoolEntry("/kniga/font/italic",FALSE)
    );
    background=QColor(
    settings.readEntry("/kniga/color/background", "white")
    );
    foreground=QColor(
    settings.readEntry("/kniga/color/foreground", "blue")
    );
    usePaper=settings.readBoolEntry("/kniga/paper/usepaper",FALSE);
    paper=settings.readEntry("kniga/paper/path","");
    defaultCode=settings.readEntry("/kniga/encoding/default","UTF-8");
    autoCodeRus=settings.readBoolEntry("/kniga/encoding/autodetectRus",FALSE);
    autoCodeFinnish=settings.readBoolEntry("/kniga/encoding/autodetectFinnish",
    FALSE);
    autoCodePortuguese=settings.readBoolEntry("/kniga/encoding/autodetectPortuguese",FALSE);
    autoCodeSlovak=settings.readBoolEntry("/kniga/encoding/autodetectSlovak",
    FALSE);
    autoCodeSlovenian=settings.readBoolEntry("/kniga/encoding/autodetectSlovenian",
    FALSE);
    autoCodeTurkish=settings.readBoolEntry("/kniga/encoding/autodetectTurkish",
    FALSE);
    autoCodeSpanish=settings.readBoolEntry("/kniga/encoding/autodetectSpanish",
    FALSE);
    autoCodePolish=settings.readBoolEntry("/kniga/encoding/autodetectPolish",
    FALSE);
    autoCodeLatvian=settings.readBoolEntry("/kniga/encoding/autodetectLatvian",
    FALSE);
    autoCodeGerman=settings.readBoolEntry("/kniga/encoding/autodetectGerman",
    FALSE);
    autoCodeSerbian=settings.readBoolEntry("/kniga/encoding/autodetectSerbian",
    FALSE);
    autoCodeUa=settings.readBoolEntry("/kniga/encoding/autodetectUa",FALSE);
    
    confirmOnExit=settings.readBoolEntry("/kniga/confirms/OnExit",TRUE);
    confirmOnBmDel=settings.readBoolEntry("/kniga/confirms/OnBmDel",TRUE);
    confirmOnCloseAll=settings.readBoolEntry("/kniga/confirms/OnCloseAll",TRUE);

    autoscroll=settings.readBoolEntry("/kniga/misc/autoscroll",FALSE);
    scroll=settings.readNumEntry("/kniga/misc/scroll",100);

    recentFiles[0]=settings.readEntry("/kniga/recentFiles/1",
        QString(DOC_PATH)+QString("README"));
    recentFiles[1]=settings.readEntry("/kniga/recentFiles/2",
    QString(DOC_PATH)+QString("ChangeLog"));
    recentFiles[2]=settings.readEntry("/kniga/recentFiles/3",
    QString(DOC_PATH)+QString("TODO"));
    recentFiles[3]=settings.readEntry("/kniga/recentFiles/4",
    QString(DOC_PATH)+QString("AUTHORS"));
    recentFiles[4]=settings.readEntry("/kniga/recentFiles/5",
    QString(DOC_PATH)+QString("COPYING"));
    int n=settings.readNumEntry("/kniga_bookmark/general/count",0);
    for (int i=0; i<n; i++) {
        bm[settings.readEntry(QString("/kniga_bookmark/%1/caption").arg(i),"")]=
        BookmarkEntry(
        settings.readEntry(QString("/kniga_bookmark/%1/filename").arg(i),""),
        settings.readNumEntry(QString("/kniga_bookmark/%1/position").arg(i),0),
        settings.readEntry(QString("/kniga_bookmark/%1/encoding").arg(i),
        defaultCode));
    };
}

void Settings::saveSettings()
{
    QSettings settings;
    settings.setPath( "Vlad Svetlichny", "Kniga" );
    
    int n=bm.count();
    settings.writeEntry("/kniga/font/name",font.family());
    settings.writeEntry("/kniga/font/size",font.pointSize());
    settings.writeEntry("/kniga/font/weight",font.weight());
    settings.writeEntry("/kniga/font/italic",font.italic());
    settings.writeEntry("/kniga/misc/autoscroll",autoscroll);
    settings.writeEntry("/kniga/misc/scroll",scroll);
    settings.writeEntry("/kniga/color/background",background.name());
    settings.writeEntry("/kniga/color/foreground",foreground.name());
    settings.writeEntry("/kniga/paper/path",paper);
    settings.writeEntry("/kniga/paper/usepaper",usePaper);
    settings.writeEntry("/kniga/encoding/default",defaultCode);
    settings.writeEntry("/kniga/encoding/autodetectRus",autoCodeRus);
    settings.writeEntry("/kniga/encoding/autodetectUa",autoCodeUa);
    settings.writeEntry("/kniga/encoding/autodetectFinnish",autoCodeFinnish);
    settings.writeEntry("/kniga/encoding/autodetectPortuguese",autoCodePortuguese);
    settings.writeEntry("/kniga/encoding/autodetectSlovak",autoCodeSlovak);
    settings.writeEntry("/kniga/encoding/autodetectSlovenian",autoCodeSlovenian);
    settings.writeEntry("/kniga/encoding/autodetectTurkish",autoCodeTurkish);
    settings.writeEntry("/kniga/encoding/autodetectSpanish",autoCodeSpanish);
    settings.writeEntry("/kniga/encoding/autodetectPolish",autoCodePolish);
    settings.writeEntry("/kniga/encoding/autodetectLatvian",autoCodeLatvian);
    settings.writeEntry("/kniga/encoding/autodetectGerman",autoCodeGerman);
    settings.writeEntry("/kniga/encoding/autodetectSerbian",autoCodeSerbian);
    settings.writeEntry("/kniga/confirms/OnExit",confirmOnExit);
    settings.writeEntry("/kniga/confirms/OnBmDel",confirmOnBmDel);
    settings.writeEntry("/kniga/confirms/OnCloseAll",confirmOnCloseAll);
    settings.writeEntry("/kniga/recentFiles/1",recentFiles[0]);
    settings.writeEntry("/kniga/recentFiles/2",recentFiles[1]);
    settings.writeEntry("/kniga/recentFiles/3",recentFiles[2]);
    settings.writeEntry("/kniga/recentFiles/4",recentFiles[3]);
    settings.writeEntry("/kniga/recentFiles/5",recentFiles[4]);
    settings.writeEntry("/kniga/win_geometry/winWidth",winWidth);
    settings.writeEntry("/kniga/win_geometry/winHeight",winHeight);
    settings.writeEntry("/kniga/win_geometry/fullscreen",fullscreen);
    settings.writeEntry("/kniga/sidebar/isVisible",sbVisible);
    settings.writeEntry("/kniga/sidebar/currentWidget",sbCurrent);
    settings.writeEntry("/kniga/sidebar/width",sbWidth);
    settings.writeEntry("/kniga_bookmark/general/count",n);
    for (int i=0; i<n; i++) {
        settings.writeEntry(QString("/kniga_bookmark/%1/caption").arg(i),
            bm.keys()[i]);
        settings.writeEntry(QString("/kniga_bookmark/%1/filename").arg(i),
            bm.values()[i].file);
        settings.writeEntry(QString("/kniga_bookmark/%1/position").arg(i),
            bm.values()[i].position);
        settings.writeEntry(QString("/kniga_bookmark/%1/encoding").arg(i),
            bm.values()[i].enc);
    }
}
